import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.util.LinkedList;
import java.util.PriorityQueue;

/**
 * Eine Klasse zur Beschreibung des Graphen.
 * 
 * @author claus albowski
 * @version Juni 2015
 */
public class Graph
{
    private int breite, hoehe, anzahl;
    private String farbe;
    // Beispielgraph nach Gallenbacher
    private String[] knotenNamen =
        {"A","B","C","D","E","F","G","H","I","K","L","M","N","O","P","X","Y","Z"};
    private String[][] kantenDaten =
        {{"A","B","69"},{"A","D","36"},{"A","M","36"},{"A","N","22"},
            {"B","A","69"},{"B","D","64"},{"B","Z","32"},{"B","H","30"},{"B","I","34"},{"B","X","26"},
            {"C","I","40"},{"C","M","31"},{"C","X","23"},
            {"D","A","36"},{"D","B","64"},{"D","L","95"},{"D","N","20"},
            {"E","F","79"},{"E","G","60"},{"E","K","31"},{"E","O","102"},
            {"F","E","79"},{"F","K","29"},{"F","O","14"},{"F","P","57"},
            {"G","E","60"},{"G","K","58"},{"G","L","58"},{"G","Y","30"},
            {"H","B","30"},{"H","I","65"},{"H","K","31"},{"H","L","106"},{"H","P","34"},{"H","Y","29"},
            {"I","B","34"},{"I","C","40"},{"I","H","65"},{"I","M","43"},{"I","P","55"},
            {"K","E","31"},{"K","F","29"},{"K","G","58"},{"K","H","31"},{"K","P","25"},{"K","Y","20"},
            {"L","D","95"},{"L","G","58"},{"L","H","106"},{"L","Z","40"},
            {"M","A","36"},{"M","C","31"},{"M","I","43"},{"M","X","12"},
            {"N","A","22"},{"N","D","20"},{"N","X","29"},{"N","Z","30"},
            {"O","E","102"},{"O","F","14"},{"O","P","91"},
            {"P","F","57"},{"P","H","34"},{"P","I","55"},{"P","K","25"},{"P","O","91"},
            {"X","B","26"},{"X","C","23"},{"X","M","12"},{"X","N","29"},
            {"Y","G","30"},{"Y","H","29"},{"Y","K","20"},{"Y","Z","23"},
            {"Z","B","32"},{"Z","L","40"},{"Z","N","30"},{"Z","Y","23"}};

    /**
     * Erzeuge eine neuen entsprechend dem Gallenbacher-Graphen
     */
    public Graph()
    {
        for (int i=0; i<knotenNamen.length; i++) new Knoten(knotenNamen[i]);
        for (int i=0; i<kantenDaten.length; i++) 
            new Kante(Knoten.gibKnoten(kantenDaten[i][0]),
                Knoten.gibKnoten(kantenDaten[i][1]),
                Integer.parseInt(kantenDaten[i][2]));
        // Eine Loesung: I -> P -> K -> F -> O, Laenge 123.0
    }

    /**
     * Erzeuge einen Testgraphen mit 5 oder 8 Knoten.
     */
    public Graph(int anzahl)
    {

        if (anzahl==5)
        {
            this.anzahl = 5;
            new Knoten("A");
            new Knoten("B");
            new Knoten("C");
            new Knoten("D");
            new Knoten("E");
            new Kante(Knoten.gibKnoten("A"),Knoten.gibKnoten("B"),20);
            new Kante(Knoten.gibKnoten("B"),Knoten.gibKnoten("A"),20);
            new Kante(Knoten.gibKnoten("A"),Knoten.gibKnoten("C"),25);
            new Kante(Knoten.gibKnoten("C"),Knoten.gibKnoten("A"),25);
            new Kante(Knoten.gibKnoten("B"),Knoten.gibKnoten("D"),30);
            new Kante(Knoten.gibKnoten("D"),Knoten.gibKnoten("B"),30);
            new Kante(Knoten.gibKnoten("B"),Knoten.gibKnoten("E"),15);
            new Kante(Knoten.gibKnoten("E"),Knoten.gibKnoten("B"),15);
            new Kante(Knoten.gibKnoten("C"),Knoten.gibKnoten("E"),25);
            new Kante(Knoten.gibKnoten("E"),Knoten.gibKnoten("C"),25);
            // Eine Loesung: A -> B -> E, Laenge 35.0 
        }
        else
        {
            this.anzahl = 8;
            new Knoten("A");
            new Knoten("B");
            new Knoten("C");
            new Knoten("D");
            new Knoten("E");
            new Knoten("F");
            new Knoten("G");
            new Knoten("H");
            new Kante(Knoten.gibKnoten("A"),Knoten.gibKnoten("B"),10);

            new Kante(Knoten.gibKnoten("B"),Knoten.gibKnoten("A"),10);
            new Kante(Knoten.gibKnoten("B"),Knoten.gibKnoten("C"),10);
            new Kante(Knoten.gibKnoten("B"),Knoten.gibKnoten("H"),3);
            new Kante(Knoten.gibKnoten("B"),Knoten.gibKnoten("G"),5);

            new Kante(Knoten.gibKnoten("C"),Knoten.gibKnoten("B"),10);
            new Kante(Knoten.gibKnoten("C"),Knoten.gibKnoten("D"),6);
            new Kante(Knoten.gibKnoten("C"),Knoten.gibKnoten("E"),4);
            new Kante(Knoten.gibKnoten("C"),Knoten.gibKnoten("H"),8);

            new Kante(Knoten.gibKnoten("D"),Knoten.gibKnoten("C"),6);
            new Kante(Knoten.gibKnoten("D"),Knoten.gibKnoten("E"),3);

            new Kante(Knoten.gibKnoten("E"),Knoten.gibKnoten("C"),4);
            new Kante(Knoten.gibKnoten("E"),Knoten.gibKnoten("D"),3);
            new Kante(Knoten.gibKnoten("E"),Knoten.gibKnoten("F"),10);

            new Kante(Knoten.gibKnoten("F"),Knoten.gibKnoten("E"),10);
            new Kante(Knoten.gibKnoten("F"),Knoten.gibKnoten("G"),8);
            new Kante(Knoten.gibKnoten("F"),Knoten.gibKnoten("H"),6);

            new Kante(Knoten.gibKnoten("G"),Knoten.gibKnoten("B"),5);
            new Kante(Knoten.gibKnoten("G"),Knoten.gibKnoten("F"),8);
            new Kante(Knoten.gibKnoten("G"),Knoten.gibKnoten("H"),5);

            new Kante(Knoten.gibKnoten("H"),Knoten.gibKnoten("B"),3);
            new Kante(Knoten.gibKnoten("H"),Knoten.gibKnoten("C"),8);
            new Kante(Knoten.gibKnoten("H"),Knoten.gibKnoten("F"),6);
            new Kante(Knoten.gibKnoten("H"),Knoten.gibKnoten("G"),5);
            // Eine Loesung: A -> B -> C -> D, Laenge 26.0
        }
    }

}

