import java.util.ArrayList;
import java.util.Iterator;
import java.util.PriorityQueue;

/**
 * Die Klasse Knoten repraesentiert Knoten in einem abstrakten Graphen.
 * Sie stellt Methoden zur Verwaltung aller Knoten bereit.
 * 
 * Es werden nur vom Knoten ausgehende Kanten verwaltet
 * 
 * Erweiterung fuer Dijkstra um Bewertung
 * und um einen Namen zur Identifikation
 * 
 * @author claus albowski
 * @version Juni 2015
 */
public class Knoten
implements Comparable
{
    // Klassenvariable
    private static ArrayList<Knoten> knotenListe = new ArrayList();
    // Instanzvariable
    private ArrayList<Kante> kantenListe;
    private double bewertung;
    private boolean istExpandiert;
    private String name;
    
    /**
     * Konstruktor mit Uebergabe eines Namens des Knoten
     */
    public Knoten(String name) {
        kantenListe = new ArrayList();
        bewertung = -1.0; // kennzeichnet: unbewertet
        istExpandiert = false;
        this.name = name;
        knotenListe.add(this);
    }
        
    /**
     * gibt zum Namen des Knoten das Knoten-Objekt zurueck;
     * gibt es den Namen nicht, wird null zurueckgegeben
     */
    public static Knoten gibKnoten(String name)
    {
        for (Knoten knoten : knotenListe)
        if (knoten.gibNamen()==name) return knoten;
        return null;
    }
    
    /**
     * set-Methode fuer die Bewertung
     */
    public void setzeBewertung(double bewertung)
    {
        this.bewertung = bewertung;
    }
    
    /**
     * get-Methode fuer die Bewertung
     */
    public double gibBewertung() { return bewertung; }
    
    /**
     * sondierende Methode fuer die Frage ob bewertet
     */
    public boolean istBewertet() { return !(bewertung<0.0); }
    
    // Fuer das Interface  Comparable notwendige Methode:
    public int compareTo(Object k) {
        if (bewertung<((Knoten)k).gibBewertung()) return -1;
        else if (bewertung>((Knoten)k).gibBewertung()) return 1;
        else return 0;
    }
    
    /**
     * set-Methode fuer die Eigenschaft, expandiert zu sein
     */
    public void wirdExpandiert() { istExpandiert = true; }
    
    /**
     * get-Methode fuer die expandiert-Eigenschaft
     */
    public boolean istExpandiert() { return istExpandiert; }
    
    /**
     * get-Methode fuer den Namen
     */
    public String gibNamen() { return name; }
    
    /**
     * nimmt eine neue Kante auf.
     */
    public void neueKante(Kante k){
        kantenListe.add(k);
    }
    
    /**
     * entfernt eine vorhandene Kante.
     */
    public void entferneKante(Kante k){
        kantenListe.remove(k);
    }
    
    /**
     * gibt die von diesem Knoten ausgehenden Kanten zurueck.
     */
    public ArrayList<Kante> gibNachfolgeKanten(){
        return kantenListe;
    }
}
